@@ -1,4 +1,4 @@
-// $Id: wxCatapultFrm.cpp,v 1.66 2005/06/23 21:09:12 manuelbi Exp $
+// $Id: wxCatapultFrm.cpp 9752 2009-05-04 19:17:28Z manuelbi $
 // ----------------------------------------------------------------------------
 // headers
 // ----------------------------------------------------------------------------
@@ -23,6 +23,7 @@
 #include "InputPage.h"
 #include "Version.h"
 #include "AboutDlg.h"
+#include "CheckConfigsDlg.h"
 #ifdef __WXMSW__
 #include "openMSXWindowsController.h"
 #else
@@ -32,6 +33,8 @@
 
 #define unisprintf sprintf
 
+class NoOpenMSXBinaryException : public std::exception {};
+
 // ----------------------------------------------------------------------------
 // constants
 // ----------------------------------------------------------------------------
@@ -41,12 +44,14 @@ enum
 {
 	// menu items and controls
 	Catapult_Quit = 1,
+	Catapult_CheckConfigs,
 	Catapult_About,
 	Catapult_Edit_Config,
 	Catapult_Load_OpenMSX_Settings,
 	Catapult_Save_OpenMSX_Settings,
 	Catapult_Save_OpenMSX_Settings_As,
 	Catapult_Save_On_Exit,
+	Catapult_Display_Invalids
 };
 
 #define FPS_TIMER 1
@@ -64,12 +69,14 @@ enum
 IMPLEMENT_CLASS(wxCatapultFrame, wxFrame)
 BEGIN_EVENT_TABLE(wxCatapultFrame, wxFrame)
 	EVT_MENU(Catapult_Quit,  wxCatapultFrame::OnMenuQuit)
+	EVT_MENU(Catapult_CheckConfigs, wxCatapultFrame::OnMenuCheckConfigs)
 	EVT_MENU(Catapult_About, wxCatapultFrame::OnMenuAbout)
 	EVT_MENU(Catapult_Edit_Config, wxCatapultFrame::OnMenuEditConfig)
 	EVT_MENU(Catapult_Save_OpenMSX_Settings, wxCatapultFrame::OnMenuSaveSettings)
 	EVT_MENU(Catapult_Save_OpenMSX_Settings_As,wxCatapultFrame::OnMenuSaveSettingsAs)
 	EVT_MENU(Catapult_Load_OpenMSX_Settings,wxCatapultFrame::OnMenuLoadSettings)
 	EVT_MENU(Catapult_Save_On_Exit,wxCatapultFrame::OnMenuSaveOnExit)
+	EVT_MENU(Catapult_Display_Invalids, wxCatapultFrame::OnMenuDisplayBroken)
 	EVT_COMMAND (-1, EVT_CONTROLLER, wxCatapultFrame::OnControllerEvent)
 	EVT_BUTTON(XRCID("Launch_AbortButton"),wxCatapultFrame::OnLaunch)
 	EVT_TIMER(FPS_TIMER, wxCatapultFrame::OnUpdateFPS)
@@ -83,8 +90,8 @@ BEGIN_EVENT_TABLE(wxCatapultFrame, wxFrame)
 	EVT_SOCKET (OPENMSX_SOCKET,wxCatapultFrame::OnSocketEvent)
 END_EVENT_TABLE()
 
-	// include icon for any non-unix version
-#if defined(__WXGTK__) || defined(__WXX11__) || defined(__WXMOTIF__) || defined(__WXMAC__) || defined(__WXMGL__)
+	// include icon if we're not compiling with Visual Studio
+#if !defined(_MSC_VER)
 #include "catapult.xpm"
 #endif
 
@@ -99,34 +106,53 @@ END_EVENT_TABLE()
 	m_focusTimer.SetOwner(this,FOCUS_TIMER);
 	m_safetyTimer.SetOwner(this,SAFETY_TIMER);
 
-#ifdef __WXMSW__	
+//	int * testke = NULL;
+//	delete testke;
+
+
+#ifdef __WXMSW__
 	m_controller = new openMSXWindowsController(this);
 #else
 	m_controller = new openMSXLinuxController(this);
 #endif
 
 	wxXmlResource::Get()->LoadFrame(this, parent, wxT("CatapultFrame"));
-	SetIcon (wxICON(catapult));
+	// use icon resources for MS Visual Studio, else the XPM
+#if defined (_MSC_VER)
+	SetIcon (wxIcon(wxT("catapult")));
+#else
+	SetIcon (catapult_xpm);
+#endif
 
 	// create menu bars
 
 	wxMenu *fileMenu = new wxMenu(wxT(""), 0);
 	settingsMenu = new wxMenu(wxT(""), 0);
+	viewMenu = new wxMenu(wxT(""), 0);
 	wxMenu *helpMenu = new wxMenu(wxT(""), 0);
 
-	fileMenu->Append(Catapult_Quit, wxT("&Quit\tCtrl-Q"), wxT("Quit this program"));
+	fileMenu->Append(Catapult_CheckConfigs, wxT("&Test MSX Hardware\tCtrl-T"), wxT("Tests which machines and extensions installed are actually working and removes non-working hardware from the lists"));
+	fileMenu->Append(Catapult_Quit, wxT("&Quit\tCtrl-Q"), wxT("Quit openMSX Catapult"));
 	settingsMenu->Append(Catapult_Edit_Config, wxT("Edit &Configuration\tCtrl-E"), wxT("Adjust Catapult Configuration"));
-	settingsMenu->Append(Catapult_Load_OpenMSX_Settings, wxT("&Load openMSX Settings..."), wxT("Load specified settings into openMSX"));
-	settingsMenu->Append(Catapult_Save_OpenMSX_Settings, wxT("&Save openMSX Settings"), wxT("Save All openMSX settings"));
+	settingsMenu->Append(Catapult_Load_OpenMSX_Settings, wxT("&Load openMSX Settings...\tCtrl-L"), wxT("Load specified settings into openMSX"));
+	settingsMenu->Append(Catapult_Save_OpenMSX_Settings, wxT("&Save openMSX Settings\tCtrl-S"), wxT("Save All openMSX settings"));
 	settingsMenu->Append(Catapult_Save_OpenMSX_Settings_As, wxT("Save openMSX Settings &As..."), wxT("Save All openMSX settings to a specified file"));
 	settingsMenu->AppendCheckItem(Catapult_Save_On_Exit, wxT("Save openMSX Settings On &Exit"), wxT("Save All openMSX settings as soon as openMSX is closed"));
+	viewMenu->AppendCheckItem(Catapult_Display_Invalids, wxT("Display Broken Configurations"), wxT("Display all machines and extension even if they don't work"));
 	helpMenu->Append(Catapult_About, wxT("&About\tCtrl-A"), wxT("Show about dialog"));
 
+	ConfigurationData * config = ConfigurationData::instance();
+
 	// now append the freshly created menu to the menu bar...
 	wxMenuBar *menuBar = new wxMenuBar();
 	menuBar->Append(fileMenu, wxT("&File"));
 	menuBar->Append(settingsMenu, wxT("&Settings"));
 	EnableSaveSettings(false);
+	menuBar->Append(viewMenu, wxT("&View"));
+	int viewFlags;
+	config->GetParameter(ConfigurationData::CD_VIEWFLAGS,&viewFlags);
+	viewMenu->Check(Catapult_Display_Invalids,((viewFlags & ConfigurationData::VF_BROKEN) != 0));
+
 	menuBar->Append(helpMenu, wxT("&Help"));
 
 	SetMenuBar(menuBar);
@@ -149,28 +175,46 @@ END_EVENT_TABLE()
 	m_videoControlPage = new VideoControlPage(m_tabControl,m_controller);
 	m_audioControlPage = new AudioControlPage(m_tabControl,m_controller);
 	m_inputPage = new InputPage(m_tabControl,m_controller);
-	
+
 	m_tabControl->AddPage(m_sessionPage,wxT("Session"),true);
 	m_tabControl->AddPage(m_miscControlPage,wxT("Misc Controls"),false);
 	m_tabControl->AddPage(m_videoControlPage,wxT("Video Controls"),false);
 	m_tabControl->AddPage(m_audioControlPage,wxT("Audio Controls"),false);
 	m_tabControl->AddPage(m_inputPage,wxT("Input Text"),false);
 	m_tabControl->AddPage(m_statusPage,wxT("Status Info"),false);
-	
-	wxWindow * tempwindow = FindWindowByName(wxT("MainWindowPanel"));
-	wxSize size = tempwindow->GetSizer()->Fit(tempwindow);
-	this->GetSizer()->SetMinSize(size);
-	this->GetSizer()->Fit(this);
-	this->GetSizer()->SetSizeHints(this);
 
 	m_launch_AbortButton = (wxButton *)FindWindowByName(wxT("Launch_AbortButton"));
 
 	SetControlsOnEnd();
 	m_launch_AbortButton->Enable(false);
-	wxString cmd;
-	ConfigurationData::instance()->GetParameter(ConfigurationData::CD_EXECPATH, cmd);
-	m_controller->StartOpenMSX(cmd,true);
 	m_settingsfile = wxT("");
+	// Now, let's find out if we have a path to openMSX
+	bool configOK = true;
+	if (!config->HaveRequiredSettings())
+	{
+		configOK = EditConfig(true);
+	}
+	if (configOK)
+	{
+		wxString cmd;
+		config->GetParameter(ConfigurationData::CD_EXECPATH, cmd);
+		if (!(m_controller->StartOpenMSX(cmd,true)))
+		{
+			configOK = EditConfig(true);
+		}
+		if (configOK)
+		{
+			m_sessionPage->SetupHardware(true, viewMenu->IsChecked(Catapult_Display_Invalids));
+			m_sessionPage->FixLayout();
+			wxWindow * tempwindow = FindWindowByName(wxT("MainWindowPanel"));
+			wxSize size = tempwindow->GetSizer()->Fit(tempwindow);
+			this->GetSizer()->SetMinSize(size);
+			this->GetSizer()->Fit(this);
+			this->GetSizer()->SetSizeHints(this);
+			m_sessionPage->RestoreHistory();
+			m_sessionPage->SetupHardware(false, viewMenu->IsChecked(Catapult_Display_Invalids));
+		} else throw NoOpenMSXBinaryException();
+	} else throw NoOpenMSXBinaryException();
 }
 
 // frame destructor
@@ -186,6 +230,48 @@ void wxCatapultFrame::OnMenuQuit(wxCommandEvent& event)
 	Close(TRUE);
 }
 
+void wxCatapultFrame::OnMenuCheckConfigs(wxCommandEvent& event)
+{
+	CheckConfigs();
+}
+
+void wxCatapultFrame::CheckConfigs()
+{
+	m_sessionPage->SetupHardware(false,true);
+	wxString cmd;
+	ConfigurationData::instance()->GetParameter(ConfigurationData::CD_EXECPATH, cmd);
+	wxArrayString machines = m_sessionPage->GetDetectedMachines();
+	wxArrayString extensions = m_sessionPage->GetDetectedExtensions();
+	CheckConfigsDlg dlg(this);
+	dlg.CenterOnParent();
+	if (dlg.ShowModal(cmd,machines,extensions) == wxID_OK){
+		ConfigurationData * config = ConfigurationData::instance();
+		wxString machineString = wxT("");
+
+		unsigned int j;
+		for (j=0;j<machines.GetCount();j++) {
+			machineString += machines[j];
+			machineString += wxT("::");
+
+		}
+		wxString extensionString = wxT("");
+		for (j=0;j<extensions.GetCount();j++){
+			extensionString += extensions[j];
+			extensionString += wxT("::");
+		}
+		config->SetParameter(ConfigurationData::CD_MACHINES,machineString);
+		config->SetParameter(ConfigurationData::CD_EXTENSIONS,extensionString);
+		bool result;
+		result = ConfigurationData::instance()->SaveData();
+#if !OPENMSX_DEMO_CD_VERSION
+		if (!result) {
+			wxMessageBox (wxT("Error saving configuration data"));
+		}
+#endif
+	}
+	m_sessionPage->SetupHardware(false, viewMenu->IsChecked(Catapult_Display_Invalids));
+}
+
 void wxCatapultFrame::OnMenuAbout(wxCommandEvent& event)
 {
 	// called when help - about is picked from the menu or toolbar
@@ -194,14 +280,14 @@ void wxCatapultFrame::OnMenuAbout(wxCommandEvent& event)
 	wxStaticText * description = (wxStaticText *)FindWindowByName(wxT("AboutProductDescriptionLabel"));
 	wxStaticText * message = (wxStaticText *)FindWindowByName(wxT("AboutMessageLabel"));
 	wxString msg;
-	msg.Printf(Version::FULL_VERSION 
+	msg.Printf(Version::FULL_VERSION
 #if OPENMSX_DEMO_CD_VERSION
-		+wxT("(CD Version)") 	
+		+wxT("(CD Version)")
 #endif
 	);
-	version->SetLabel(msg); 
+	version->SetLabel(msg);
 	description->SetLabel(wxT("The official GUI for openMSX"));
-	message->SetLabel(wxT("\251 2003-2005 The openMSX Team\n<openmsx-devel@lists.sourceforge.net>\n"));
+	message->SetLabel(wxT("\251 2003-2007 The openMSX Team\n<openmsx-devel@lists.sourceforge.net>\n"));
 #ifdef __WXMSW__
 		dlg.SetSize(-1,-1,400,307,0);
 #else
@@ -213,8 +299,13 @@ void wxCatapultFrame::OnMenuAbout(wxCommandEvent& event)
 
 void wxCatapultFrame::OnMenuEditConfig(wxCommandEvent& event)
 {
-	wxString cmd;
-	CatapultConfigDlg dlg;
+	EditConfig(false);
+}
+
+bool wxCatapultFrame::EditConfig(bool fatalIfFails)
+{
+	bool retval = true;
+	CatapultConfigDlg dlg(this, m_controller);
 	dlg.Center();
 	if (dlg.ShowModal() == wxID_OK) {
 		bool result = ConfigurationData::instance()->SaveData();
@@ -223,11 +314,11 @@ void wxCatapultFrame::OnMenuEditConfig(wxCommandEvent& event)
 			wxMessageBox (wxT("Error saving configuration data!"));
 		}
 #endif
-		m_sessionPage->SetupHardware(false);
-		ConfigurationData::instance()->GetParameter(ConfigurationData::CD_EXECPATH, cmd);
-		
-		m_controller->StartOpenMSX(cmd,true);
+		m_sessionPage->SetupHardware(false,false);
+	} else {
+		retval = !fatalIfFails;
 	}
+	return retval;
 }
 
 void wxCatapultFrame::OnMenuLoadSettings(wxCommandEvent &event)
@@ -246,7 +337,7 @@ void wxCatapultFrame::OnMenuLoadSettings(wxCommandEvent &event)
 		settingsfile = filedlg.GetPath();
 		if (m_controller->IsOpenMSXRunning()){
 			m_controller->WriteCommand(wxString(wxT("load_settings ")) + m_sessionPage->ConvertPath(settingsfile,true));
-		}		
+		}
 		else{
 			m_settingsfile = settingsfile;
 		}
@@ -273,7 +364,7 @@ void wxCatapultFrame::OnMenuSaveSettingsAs (wxCommandEvent & event)
 	{
 		settingsfile = filedlg.GetPath();
 		m_controller->WriteCommand(wxString(wxT("save_settings ")) + m_sessionPage->ConvertPath(settingsfile,true));
-	}	
+	}
 }
 
 void wxCatapultFrame::OnMenuSaveOnExit(wxCommandEvent &event)
@@ -286,6 +377,21 @@ void wxCatapultFrame::OnMenuSaveOnExit(wxCommandEvent &event)
 	}
 }
 
+void wxCatapultFrame::OnMenuDisplayBroken (wxCommandEvent & event)
+{
+	ConfigurationData * config = ConfigurationData::instance();
+	int viewFlags;
+	config->GetParameter(ConfigurationData::CD_VIEWFLAGS,&viewFlags);
+	m_sessionPage->SetupHardware(false, viewMenu->IsChecked(Catapult_Display_Invalids));
+	if (viewMenu->IsChecked(Catapult_Display_Invalids)){
+		config->SetParameter(ConfigurationData::CD_VIEWFLAGS,(long)(viewFlags | ConfigurationData::VF_BROKEN));
+	}
+	else{
+		config->SetParameter(ConfigurationData::CD_VIEWFLAGS,(long)(viewFlags & ~ConfigurationData::VF_BROKEN));
+	}
+	config->SaveData();
+}
+
 
 void wxCatapultFrame::EnableSaveSettings(bool enabled)
 {
@@ -295,12 +401,12 @@ void wxCatapultFrame::EnableSaveSettings(bool enabled)
 
 void wxCatapultFrame::OnMenuOpen(wxMenuEvent &event)
 {
-	m_tempStatus = GetStatusBar()->GetStatusText(0);	
+	m_tempStatus = GetStatusBar()->GetStatusText(0);
 }
 
 void wxCatapultFrame::OnMenuClose(wxMenuEvent &event)
 {
-	SetStatusText(m_tempStatus,0);	
+	SetStatusText(m_tempStatus,0);
 }
 
 void wxCatapultFrame::OnMenuHighlight(wxMenuEvent &event)
@@ -309,17 +415,23 @@ void wxCatapultFrame::OnMenuHighlight(wxMenuEvent &event)
 		wxFrame::OnMenuHighlight(event);
 	}
 	else{
-		SetStatusText(m_tempStatus,0);	
+		SetStatusText(m_tempStatus,0);
 	}
 }
 
 void wxCatapultFrame::OnLaunch(wxCommandEvent& event)
 {
-	unsigned int i;
 	if (m_launch_AbortButton->GetLabel().IsSameAs(wxT("Stop"))) {
-		m_controller->WriteCommand(wxT("quit"));
+		m_controller->WriteCommand(wxT("exit"));
 		return;
 	}
+	// check again if openMSX can be found
+	ConfigurationData * config = ConfigurationData::instance();
+	if (!config->HaveRequiredSettings())
+	{
+		if (!EditConfig(true)) return;
+	}
+
 	m_launch_AbortButton->SetLabel(wxT("Stop"));
 	m_launch_AbortButton->Enable(false);
 	m_safetyTimer.Start(2000,true); // max 2 seconds disable (whatever happens)
@@ -328,38 +440,48 @@ void wxCatapultFrame::OnLaunch(wxCommandEvent& event)
 	wxArrayString media;
 	wxArrayString patches[5];
 	wxArrayString types;
-	
+
 	m_sessionPage->getHardware(hardware);
 	m_sessionPage->getMedia(media);
 	m_sessionPage->getPatches(patches);
 	m_sessionPage->getTypes(types);
 
-	Enable(false); // Disable this frame only after getting the selections
-	
 	wxString cmd;
 	ConfigurationData::instance()->GetParameter(ConfigurationData::CD_EXECPATH, cmd);
-	if (hardware[0] != wxT(" <default> "))
-	{
-		cmd += wxT(" -machine ") + hardware[0];
-	}
 	if (m_settingsfile != wxT("")){
 		cmd += wxT(" -setting ") + m_sessionPage->ConvertPath(m_settingsfile,true);
 	}
+	if (hardware[0] != wxT(" <default> ")) // Ooooww.... we're using representation here as useful data! :(
+	{
+		cmd += wxT(" -machine ") + hardware[0];
+	}
+
+	// EXTENSIONS start from index 1 in hardware!
+
+	// Here's a hack to make sure that slotexpander is always mentioned first. Note that this will break terribly if someone renames the slotexpander extension. Hence the 'hack' remark.
+	unsigned int i = hardware.Index(wxT("slotexpander"), false);
+
+	if ((int)i != wxNOT_FOUND) {
+		// put it at item 1
+		hardware.RemoveAt(i);
+		hardware.Insert(wxT("slotexpander"), 1);
+	}
+
 	if (hardware.GetCount() > 1) {
 		for (i=1;i<hardware.GetCount();i++) {
 			cmd += wxT(" -ext ") + hardware[i];
 		}
 	}
-	wxString parmname [5]={wxT("diska"),wxT("diskb"),wxT("carta"),wxT("cartb"),wxT("cassetteplayer")};
+	wxString parmname [5]={wxT("diska"),wxT("diskb"),wxT("cart"),wxT("cart"),wxT("cassetteplayer")};
 	FOREACH(i,parmname) {
 		if (!media[i].IsEmpty()) {
 			cmd += wxT(" -") + parmname[i] +wxT(" \"") + media[i] + wxT("\"");
 			if (!types[i].IsEmpty()){
 				cmd += wxT(" -romtype ") + types[i];
 			}
-			int count = patches[i].GetCount();
+			size_t count = patches[i].GetCount();
 			if (count != 0){
-				for (int j=0;j<count;j++){
+				for (size_t j=0;j<count;j++){
 					cmd += wxT(" -ips \"") + patches[i].Item(j) + wxT("\"");
 				}
 			}
@@ -367,6 +489,11 @@ void wxCatapultFrame::OnLaunch(wxCommandEvent& event)
 	}
 	m_sessionPage->UpdateSessionData();
 	m_statusPage->m_outputtext->Clear();
+
+	Enable(false); // Disable this frame only after getting the selections (so, also AFTER UpdateSessionData!)
+
+	//std::cerr << "Generic command is: " << std::string((const char*)(wxConvUTF8.cWX2MB(cmd))) << std::endl;
+
 	m_controller->StartOpenMSX(cmd);
 	SetControlsOnLaunch();
 }
@@ -389,7 +516,7 @@ void wxCatapultFrame::SetControlsOnEnd()
 
 void wxCatapultFrame::OnControllerEvent(wxCommandEvent &event)
 {
-	m_controller->HandleMessage(event);	
+	m_controller->HandleMessage(event);
 }
 
 void wxCatapultFrame::OnSocketEvent(wxSocketEvent & event)
@@ -413,18 +540,18 @@ void wxCatapultFrame::StopTimers()
 void wxCatapultFrame::SetFPSdisplay(wxString val)
 {
 	double valfl = strtod((const char*) (wxConvUTF8.cWX2MB(val)),NULL);
-	val.sprintf(wxT("%2.1f"),valfl);	
+	val.sprintf(wxT("%2.1f"),valfl);
 	SetStatusText(val +wxT(" fps"),1);
 }
 
 void wxCatapultFrame::OnUpdateFPS(wxTimerEvent& event)
 {
-	m_controller->WriteCommand(m_controller->GetInfoCommand(wxT("fps")));
+	m_controller->WriteCommand(wxT("openmsx_info fps"));
 }
 
 void wxCatapultFrame::OnEnableMainWindow(wxTimerEvent & event)
 {
-	EnableMainWindow();	
+	EnableMainWindow();
 }
 
 void wxCatapultFrame::EnableMainWindow ()
@@ -454,10 +581,10 @@ void wxCatapultFrame::OnChangePage(wxNotebookEvent &event)
 	int newPageNr = event.GetSelection();
 	if (oldPageNr != -1) {
 		page = (CatapultPage *)m_tabControl->GetPage(oldPageNr);
-		page->HandleFocusChange(m_currentFocus,NULL);	
+		page->HandleFocusChange(m_currentFocus,NULL);
 	}
 	if (newPageNr != -1) {
-		page = (CatapultPage *)m_tabControl->GetPage(newPageNr);	
+		page = (CatapultPage *)m_tabControl->GetPage(newPageNr);
 		wxWindow * newfocus = page->FindFocus();
 		page->HandleFocusChange(NULL,newfocus);
 		m_currentFocus = newfocus;
@@ -468,7 +595,7 @@ void wxCatapultFrame::OnDeselectCatapult(wxActivateEvent & event)
 {
 	int selectedPage = m_tabControl->GetSelection();
 	if (selectedPage != -1) {
-		CatapultPage * page = (CatapultPage *)m_tabControl->GetPage(selectedPage);	
+		CatapultPage * page = (CatapultPage *)m_tabControl->GetPage(selectedPage);
 		page->HandleFocusChange(m_currentFocus,NULL);
 	}
 	m_currentFocus = NULL;
